@@ -3,6 +3,30 @@
 package Algorithm::Combinatorics::C;
 
 use Inline C => <<'END_OF_C_CODE';
+
+/**
+ * These subroutines implement the actual iterators.
+ * 
+ * The real combinatorics are done in-place on a private array of indices
+ * that is guaranteed to hold IVs. Once the next tuple has been computed
+ * on that array the corresponding slice of data is copied into out.
+ *
+ * All the subroutines return -1 when there are no further tuples, or
+ * the index of the element that was updated. I leave this a bit ambiguous
+ * on purpose because by now we are only using == -1 and the contract may
+ * change in future versions.
+ *
+ * As of this version variations() maintain a private hash table to be
+ * able to keep track of the currently used indices, that is, the elements
+ * of the indices array. This is used to be able to check efficiently whether
+ * they are free to jump over them otherwise.
+ *
+ */
+
+
+#define UPDATE(av, i, n) (sv_setiv(*av_fetch(av, i, 0), n))
+
+
 int __next_combination(AV* indices, AV* data, AV* out)
 {
     int i, j;
@@ -18,8 +42,8 @@ int __next_combination(AV* indices, AV* data, AV* out)
         if (n < i + offset) {
              sv_setiv(index, ++n);
              for (j = i+1; j <= len_indices; ++j)
-                  sv_setiv(*av_fetch(indices, j, 0), ++n);
-             __slice(i, indices, data, out);
+                  UPDATE(indices, j, ++n);
+             __slice_and_copy(i, indices, data, out);
              return i;
         }
     }
@@ -27,6 +51,7 @@ int __next_combination(AV* indices, AV* data, AV* out)
     return -1;
 }
 
+
 int __next_combination_with_repetition(AV* indices, AV* data, AV* out)
 {
     int i, j;
@@ -40,8 +65,8 @@ int __next_combination_with_repetition(AV* indices, AV* data, AV* out)
         if (n < max_n) {
              ++n;
              for (j = i; j <= len_indices; ++j)
-                  sv_setiv(*av_fetch(indices, j, 0), n);
-             __slice(i, indices, data, out);
+                  UPDATE(indices, j, n);
+             __slice_and_copy(i, indices, data, out);
              return i;
         }
     }
@@ -49,6 +74,7 @@ int __next_combination_with_repetition(AV* indices, AV* data, AV* out)
     return -1;
 }
 
+
 int __next_variation(HV* used, AV* indices, AV* data, AV* out)
 {
     int i, j;
@@ -75,13 +101,13 @@ int __next_variation(HV* used, AV* indices, AV* data, AV* out)
                             ++n;
                             he = hv_fetch_ent(used, newSViv(n), 0, 0);
                             if (he == NULL) {
-                                 av_store(indices, j, newSViv(n));
+	                             UPDATE(indices, j, n);
                                  hv_store_ent(used, newSViv(n), newSViv(j), 0);
                                  break;
                             }
                        }
                   }
-                  __slice(i, indices, data, out);
+                  __slice_and_copy(i, indices, data, out);
                   return i;
              }
         }
@@ -90,6 +116,7 @@ int __next_variation(HV* used, AV* indices, AV* data, AV* out)
     return -1;
 }
 
+
 int __next_variation_with_repetition(AV* indices, AV* data, AV* out)
 {
     int i;
@@ -102,7 +129,7 @@ int __next_variation_with_repetition(AV* indices, AV* data, AV* out)
         index = *av_fetch(indices, i, 0);
         if (SvIVX(index) < max_n) {
             sv_inc(index);
-             __slice(i, indices, data, out);
+             __slice_and_copy(i, indices, data, out);
             return i;
         }
         sv_setiv(index, 0);
@@ -112,8 +139,7 @@ int __next_variation_with_repetition(AV* indices, AV* data, AV* out)
 }
 
 
-
-void __slice(int from, AV* indices, AV* data, AV* out)
+void __slice_and_copy(int from, AV* indices, AV* data, AV* out)
 {
      int i;
      I32 len_indices;
